%CalculateGrowthEnzymeSensitivities
%  Use kinetically constrained genome-scale metabolic model to predict
%  essential metabolic enzymes. The kinetically constrained metabolic model
%  was originally developed as the metabolic sub-model of a more
%  comprehensive whole-cell model. The kinetically constrained metabolic
%  model is a flux balance analysis model with additional constraints equal
%  to the product of the enzyme copy numbers of enzyme turnover rates. The
%  enzyme copy numbers were estimated from the observed mRNA expression
%  (Weiner et al., 2003) and total protein mass (Morowitz et al., 1962).
%  The turnover numbers were curated from BRENDA, SABIO-RAK, and the
%  primary literature.
%
%  Instructions
%  1. Download whole-cell model code from http://github.com/CovertLab/WholeCell
%  2. Place this script and CalculateGrowthRateWithModifiedEnzymeExpression in the top-level directory of the whole-cell repository
%  3. Execute this script
%
%  References
%  1. Weiner 3rd, J., Zimmerman, C.U., Gohlmann, H.W., Herrmann, R., 2003.
%     Transcription profiles of the bacterium Mycoplasma pneumoniae grown
%     at different temperatures. Nucleic Acids Res. 31 (21), 63066320.
%  2. Morowitz, H.J., Tourtellotte, M.E., Guild, W.R., Castro, E., Woese,
%     C., 1962. The chemical composition and submicroscopic morphology of
%     Mycoplasma gallisepticum, avian pplo 5969. J. Mol. Biol. 4, 93103.
%
%  Author: Jonathan Karr, jonrkarr@gmail.com
%  Date: 2015-02-23

%% set warnings, path
setWarnings();
setPath();

%% import
import edu.stanford.covert.util.ComputationUtil;

%% load cached simulation object
load('data/Simulation_fitted.mat');
sim = simulation;

%% handles to sub-models and states
gene = sim.gene;
mass = sim.state('Mass');
mr = sim.state('MetabolicReaction');
met = sim.process('Metabolism');

enzGeneComp = met.enzymeGeneComposition() ~= 0;
subGeneComp = met.substrateGeneComposition() ~= 0;

nGene = numel(gene.wholeCellModelIDs);

%% run metabolic sub-model, get wild-type growth rate
%reset state
met.copyFromState();

%calculate growth rate
met.evolveState();
wtGrowth = mr.growth;

%% simulate individual gene knockouts
isGeneModeled = any(enzGeneComp, 2) | any(subGeneComp, 2);
koGrowth = NaN(nGene, 3);
exp50 = NaN(nGene, 1);
perturbation = 1e-1;
for iGene = 1:nGene
    if ~isGeneModeled(iGene)
        continue;
    end
    
    %% knockout expression
    %reset state
    met.copyFromState();
    
    %knockout enzyme
    met.enzymes(enzGeneComp(iGene, :), :) = 0;
    met.substrates(subGeneComp(iGene, :), :) = 0;
    
    %calculate growth rate
    met.evolveState();
    koGrowth(iGene, 1) = mr.growth;
    
    %% decrease expression
    %reset state
    met.copyFromState();
    
    %knockout enzyme
    met.enzymes(enzGeneComp(iGene, :), :) = (1 - perturbation) * met.enzymes(enzGeneComp(iGene, :), :);
    met.substrates(subGeneComp(iGene, :), :) = (1 - perturbation) * met.substrates(subGeneComp(iGene, :), :);
    
    %calculate growth rate
    met.evolveState();
    koGrowth(iGene, 2) = mr.growth;
    
    %% increase expression
    %reset state
    met.copyFromState();
    
    %knockout enzyme
    met.enzymes(enzGeneComp(iGene, :), :) = (1 + perturbation) * met.enzymes(enzGeneComp(iGene, :), :);
    met.substrates(subGeneComp(iGene, :), :) = (1 + perturbation) * met.substrates(subGeneComp(iGene, :), :);
    
    %calculate growth rate
    met.evolveState();
    koGrowth(iGene, 3) = mr.growth;
    
    %% calculate expression level at which growth is 50% of wild type
    hasKinRxn = any(met.fbaReactionCatalysisMatrix(:, enzGeneComp(iGene, :)), 2) & any(~isinf(met.fbaEnzymeBounds), 2);
    nKinRxn = nnz(hasKinRxn);
    if koGrowth(iGene) > wtGrowth/2 || nKinRxn == 0
        continue;
    end
    
    exp50(iGene) = fzero(@CalculateGrowthRateWithModifiedEnzymeExpression, [0 1], struct(), met, enzGeneComp, subGeneComp, wtGrowth/2, iGene);
end

%% output model statistics
isGeneEss = koGrowth(:, 1) < 1e-3 * wtGrowth;
geneHasKinData = any(enzGeneComp(:, any(met.reactionCatalysisMatrix(any(~isinf(met.enzymeBounds), 2), :), 1)), 2);

xlswrite(outPath, {
    'Statistic'                            'Count'
    'Metabolic enzymes'                    numel(met.enzymeWholeCellModelIDs)
    'Genes'                                nnz(any(enzGeneComp, 2) | any(subGeneComp, 2))
    'Metabolites'                          nnz(~any(met.substrateGeneComposition(), 1))
    'Reactions'                            numel(met.reactionWholeCellModelIDs)
    'Kinetically constrained enzymes'      nnz(any(met.reactionCatalysisMatrix(any(~isinf(met.enzymeBounds), 2), :), 1))
    'Kinetically constrained genes'        nnz(any(enzGeneComp(:, any(met.reactionCatalysisMatrix(any(~isinf(met.enzymeBounds), 2), :), 1)), 2))
    'Kinetically constrained reactions'    nnz(any(~isinf(met.enzymeBounds), 2))
    'Essential enzymes'                    nnz(any(enzGeneComp(isGeneEss, :), 1))
    'Essential genes'                      nnz(isGeneEss)
    'Reactions involving essential genes'  nnz(any(met.reactionCatalysisMatrix(:, any(enzGeneComp(isGeneEss, :), 1)), 2))
}, 'Statistics');

%% output essential genes
[~, order] = sortrows([num2cell(exp50) gene.wholeCellModelIDs], [-1 2]);
order = [
    order(ismember(order, find(geneHasKinData)) & ~isnan(exp50(order)) )
    order(ismember(order, find(geneHasKinData)) &  isnan(exp50(order)) )
    ];
exp50Cell = num2cell(exp50);
exp50Cell(isnan(exp50)) = {'Non-essential'};
xlswrite(outPath, [{
    '' '' '' 'Growth rate (fg h-1)' '' ''
    'ID' 'Name' 'N=0' sprintf('N=%d%%', round((1 - perturbation) * 100)) sprintf('N=%d%%', round((1 + perturbation) * 100)) 'Expression-50'
    }
        gene.wholeCellModelIDs(order) ...
        gene.names(order) ...    
        num2cell(max(0, koGrowth(order, 1) * 3600 * mass.cellInitialDryWeight / (1 - mass.fractionWetWeight) * 1e15)) ...
        num2cell((koGrowth(order, 2) - wtGrowth) / perturbation) ...
        num2cell((koGrowth(order, 3) - wtGrowth) / perturbation) ...
        exp50Cell(order)
    ], 'Predictived sensitivies')